from spinsrv import spinpy as sp
from spinsrv import spin
import json
import hashlib

print("THIS IS SMOKE TEST 1: IT TESTS THE CLIENTS IN src/spin/spinpy.py")

kc = sp.KeyServerHTTPClient()
resp = kc.which(
    spin.KeyWhichRequest(
        public="proteins",
        private="proteins",
    )
)
print(resp)
resp = kc.temp(
    spin.KeyTempRequest(
        public="proteins",
        private="proteins",
        duration=60000000000,
    )
)
print(resp)

dc = sp.DirServerHTTPClient()
resp = dc.lookup(
    spin.DirLookupRequest(
        public="proteins", private="proteins", citizen="proteins", path="/"
    )
)
print(resp)
resp = dc.tree(
    spin.DirTreeRequest(
        public="proteins", private="proteins", citizen="proteins", path="/", level=1
    )
)
print(resp)

resp = dc.apply(
    spin.DirApplyRequest(
        public="proteins",
        private="proteins",
        ops=[
            spin.DirOp(
                operation=spin.PutDirOperation,
                dir_entry=spin.DirEntry(
                    type=spin.EntryDir,
                    citizen="proteins",
                    path="/test",
                    sequence=spin.SeqIgnore,
                ),
            )
        ],
    )
)
print(resp)

bc = sp.BitServerHTTPClient()
data = "Asdf".encode()
resp = bc.apply(
    spin.BitApplyRequest(
        public="proteins",
        private="proteins",
        ops=[
            spin.BitOp(
                operation=spin.PutBitOperation,
                ref=spin.SHA256(data),
                bytes=data,
            )
        ],
    )
)
print(resp)



from spinsrv import spinfs

print(
    "THIS IS SMOKE TEST 2: IT TESTS (primarily directory related) functionality of src/spin/spinfs.py"
)

fs = spinfs.SpinFileSystem(
    spinfs.Config(public="proteins", private="proteins", citizen="proteins")
)

resp = fs.ls("/")
print(resp)

resp = fs.makedirs("/this/a/test")
print(resp)

resp = fs.rm_file("/this/a/test")
print(resp)

resp = fs.touch("/this/a/test.txt")
print(resp)

assert fs.exists("/this/a/test.txt")
assert fs.isfile("/this/a/test.txt")
assert fs.cat("/this/a/test.txt") == b""
resp = fs.copy("/this/a/test.txt", "/this/a/copy.txt")
print(resp)
out = {x: True for x in fs.ls("/this/a", detail=False)}
assert "/this/a/test.txt" in out, f"out was {out}"
assert "/this/a/copy.txt" in out, f"out was {out}"
resp = fs.rm("/this/a", recursive=True)
print(resp)



from spinsrv import spinfs

print(
    "THIS IS SMOKE TEST 3: IT TESTS (primarily file related) functionality of src/spin/spinfs.py"
)

fs = spinfs.SpinFileSystem(
    spinfs.Config(public="proteins", private="proteins", citizen="proteins")
)
f = fs.open("/notes.txt", "wb")
f.write(b"Hello world!")
f.close()
f = fs.open("/notes.txt", "rb")
b = f.read()
assert b == b"Hello world!", f"got {b}, wanted Hello world!"



import requests

from spinsrv import spin


class KeyServerHTTPClient(object):
    def __init__(self, session=requests.Session()):
        self.url = "https://keys.spinsrv.com"
        self.session = session

    def which(self, req: spin.KeyWhichRequest):
        url = self.url + "/which"
        return spin.KeyWhichResponse.from_json(
            self.session.post(url, json=req.to_json()).json()
        )

    def temp(self, req: spin.KeyTempRequest):
        url = self.url + "/temp"
        return spin.KeyTempResponse.from_json(
            self.session.post(url, json=req.to_json()).json()
        )


class DirServerHTTPClient(object):
    def __init__(self, session=requests.Session()):
        self.url = "https://dir.spinsrv.com"
        self.session = session

    def lookup(self, req: spin.DirLookupRequest):
        url = self.url + "/lookup"
        return spin.DirLookupResponse.from_json(
            self.session.post(url, json=req.to_json()).json()
        )

    def tree(self, req: spin.DirTreeRequest):
        url = self.url + "/tree"
        return spin.DirTreeResponse.from_json(
            self.session.post(url, json=req.to_json()).json()
        )

    def apply(self, req: spin.DirApplyRequest):
        url = self.url + "/apply"
        return spin.DirApplyResponse.from_json(
            self.session.post(url, json=req.to_json()).json()
        )


class BitServerHTTPClient(object):
    def __init__(self, session=requests.Session()):
        self.url = "https://store.spinsrv.com"
        self.session = session

    def apply(self, req: spin.BitApplyRequest):
        url = self.url + "/apply"
        return spin.BitApplyResponse.from_json(
            self.session.post(url, json=req.to_json()).json()
        )
